VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "TreeFactory"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit
Option Base 1
Private Tree As TrueVision3D.TVMesh
Private Leaves As TrueVision3D.TVMesh
Private Leaf As TrueVision3D.TVMesh
Private Const Pi = 3.14159265358979
Private Type MappedVertex
   x As Single
   y As Single
   z As Single
   tu As Single
   tv As Single
End Type
Private Type TVertex
   e As Long
   WS As Integer
   HS As Integer
   Pos As D3DVECTOR
End Type
Private T_INDEX As Long, L_INDEX As Long, F_INDEX As Long
Private TreeV() As TVertex
Private LeavesV() As TVertex
Public Enum CONST_TV_LEAVES_ORIENTATION
   TV_LEAVES_X = 0
   TV_LEAVES_Y = 1
   TV_LEAVES_Z = 2
   TV_LEAVES_XY = 3
   TV_LEAVES_ZY = 4
   TV_LEAVES_XZ = 5
   TV_Leaves_XYZ = 6
End Enum
Private Type LVertex
   Group As Integer
   Amount As Long
   Parent As Integer
   Start As Long
   End As Long
   Delay As Long
   RestLeaf As Boolean
   Size As Single
End Type
Private LeafV() As LVertex
Private Land As TrueVision3D.TVLandscape
Private FirstLeaf As Boolean
Public Sub InitTrees(Scene As TrueVision3D.TVScene, Landscape As TrueVision3D.TVLandscape)
   Set Land = Landscape
   Set Tree = Scene.CreateMeshBuilder
   Tree.SetPrimitiveType TV_TRIANGLESTRIP
   Set Leaves = Scene.CreateMeshBuilder
   Leaves.SetPrimitiveType TV_TRIANGLELIST
   Set Leaf = Scene.CreateMeshBuilder
   Leaf.SetPrimitiveType TV_TRIANGLESTRIP
   T_INDEX = 0: L_INDEX = 0: F_INDEX = 0
   ReDim TreeV(1 To 1)
   ReDim LeavesV(1 To 1)
   ReDim LeafV(1 To 1)
End Sub
Public Property Let TreeMesh(NewTreeMesh As TrueVision3D.TVMesh)
   Set Tree = NewTreeMesh
End Property
Public Property Get TreeMesh() As TrueVision3D.TVMesh
   Set TreeMesh = Tree
End Property
Public Property Let LeavesMesh(NewLeavesMesh As TrueVision3D.TVMesh)
   Set Leaves = NewLeavesMesh
End Property
Public Property Get LeavesMesh() As TrueVision3D.TVMesh
   Set LeavesMesh = Leaves
End Property
Private Sub Class_Terminate()
   Set Tree = Nothing
   Set Leaves = Nothing
   Set Leaf = Nothing
End Sub
Public Function AddTree(Texture As Long, x As Single, y As Single, z As Single, _
                       Radius As Single, Height As Single, Taper As Single, _
                       WidthSegments As Integer, HeightSegments As Integer, _
                       Optional Twist As Single = 0, Optional Noise As Single = 0) As Long
If Radius <= 0 Or Height = 0 Then
   AddTree = 0
   Exit Function
End If
Dim A As Integer
Dim b As Integer
Dim c As Integer
Dim d As Integer
Dim e As Integer
Dim TMesh() As D3DVECTOR
Dim WU As Single, HU As Single
Dim tu As Single, tv As Single
Dim TaperStep As Single
Dim TVec As D3DVECTOR

   If Radius <= 0 Then Radius = 1
   If Height <= 0 Then Height = 1
   If Taper <= 0 Then Taper = 0
   
   T_INDEX = T_INDEX + 1
   ReDim Preserve TreeV(T_INDEX)
   If T_INDEX > 1 Then
      Tree.AddFace Texture, True
   Else
      Tree.SetTexture Texture, 1
   End If
   
   With TreeV(T_INDEX)
      .e = WidthSegments * HeightSegments
      .WS = WidthSegments
      .HS = HeightSegments
      .Pos = Vector(x, y, z)
   End With
   
   HeightSegments = HeightSegments + 1

   WU = (2 * Pi) / WidthSegments
   HU = Height / HeightSegments
    
   WidthSegments = WidthSegments + 1
   
   ReDim TMesh(WidthSegments * HeightSegments)
    
   TaperStep = (Radius * Taper) / HeightSegments
   Radius = Radius * (1 - Taper)
   
   c = 1
   For A = 1 To HeightSegments
      For b = 1 To WidthSegments
         With TMesh(c)
            If Noise <> 0 Then
               .x = x + Radius * Cos((b * WU)) * (Rnd * Noise)
               .y = y + (Height - (A * HU))
               .z = z + Radius * Sin((b * WU)) * (Rnd * Noise)
            Else
               .x = x + Radius * Cos((b * WU))
               .y = y + (Height - (A * HU))
               .z = z + Radius * Sin((b * WU))
            End If
            If Twist <> 0 Then
               TVec = SetRotation(Vector(x, y, z), Vector(.x, .y, .z), 0, Twist, 0)
               .x = TVec.x
               .y = TVec.y
               .z = TVec.z
            End If
         End With
         c = c + 1
      Next b
      'Noise = Noise + 0.01
      Radius = Radius + TaperStep
      If Twist <> 0 Then Twist = Twist + (Twist / HeightSegments)
   Next A
    
   d = 0: e = 1

   For A = 1 To HeightSegments - 1
      tv = (A / HeightSegments) '* here just added
      For b = 1 To WidthSegments
         For c = d To e
            tu = (b + (WidthSegments * c) / ((WidthSegments * HeightSegments)))
            With TMesh(b + (WidthSegments * c))
               Tree.AddVertex .x, .y, .z, RGBA(1, 1, 1, 1), tu, tv
               tu = tu + (1 / (WidthSegments * HeightSegments))
               tv = tv + (1 / HeightSegments)
            End With
         Next
      Next
      d = e
      e = d + 1
   Next
   Tree.ComputeNormals
   AddTree = T_INDEX
End Function
Public Function AddLeaves(Texture As Long, Amount As Single, _
                          Altitude As Single, Optional Density As Integer = 2, _
                          Optional Orientation As CONST_TV_LEAVES_ORIENTATION = TV_LEAVES_Y, _
                          Optional TreeGroup As Long, _
                          Optional x As Single, Optional y As Single, Optional z As Single) As Long
                      
Dim n As Integer, m As Integer
Dim f As Single
Dim G As Single
Dim h As Single
Dim j As Single
Dim k As Single
Dim U As Single
Dim v As Single
Dim LMesh() As MappedVertex
Dim Dense As Single
Dim TVec As D3DVECTOR
Dim SwapSign As Integer

   SwapSign = 1

   If Amount <= 0 Then
      AddLeaves = 0
      Exit Function
   End If
   If TreeGroup Then
      If TreeGroup > UBound(TreeV) Then
         AddLeaves = 0
         Exit Function
      Else
         With TreeV(TreeGroup)
            x = .Pos.x
            z = .Pos.z
            G = .Pos.y
         End With
      End If
   End If

   If Density = 0 Then Density = 1
   
   f = (Amount / 2)
   G = G + Altitude - f
   h = Amount + G
      
   ReDim LMesh(6)
   For n = 1 To UBound(LMesh)
      Select Case n
         Case 1: j = -f: k = G: U = 1: v = 0
         Case 2: j = -f: k = h: U = 1: v = 1
         Case 3: j = f: k = G: U = 0: v = 0
         Case 4: j = f: k = h: U = 0: v = 1
         Case 5: j = -f: k = h: U = 1: v = 1
         Case 6: j = f: k = G: U = 0: v = 0
      End Select
      With LMesh(n)
         .x = x + j: .y = y + k: .z = z
         .tu = U: .tv = v
      End With
   Next
   
   Select Case Orientation
      Case Is = 0: Dense = (45 / Density)
      Case Is = 1: Dense = (180 / Density)
      Case Is = 2: Dense = (45 / Density)
   End Select
      
   L_INDEX = L_INDEX + 1
   ReDim Preserve LeavesV(L_INDEX)
   
   If L_INDEX > 1 Then
      Leaves.AddFace Texture, True
   Else
      Leaves.SetTexture Texture, 1
   End If
      
   With LeavesV(L_INDEX)
      .e = Density * 6
      .WS = Amount
      .HS = Amount + G
      .Pos = Vector(x, G, z)
   End With
      
   For n = 1 To Density
      For m = 1 To UBound(LMesh)
         With LMesh(m)
            Select Case Orientation
               Case Is = 0: TVec = SetRotation(Vector(x, G, z), Vector(.x, .y, .z), (n * Dense) * SwapSign, 0, 0)
               Case Is = 1: TVec = SetRotation(Vector(x, G, z), Vector(.x, .y, .z), 0, (n * Dense), 0)
               Case Is = 2: TVec = SetRotation(Vector(x, G, z), Vector(.x, .y, .z), 0, 0, (n * Dense) * SwapSign)
            End Select
            Leaves.AddVertex TVec.x, TVec.y, TVec.z, RGBA(1, 1, 1, 1), .tu, .tv
         End With
      Next
      SwapSign = -SwapSign
   Next
   Leaves.ComputeNormals
   AddLeaves = L_INDEX
End Function


Private Function SetRotation(Origin As D3DVECTOR, Point As D3DVECTOR, _
                        ByVal x As Single, ByVal y As Single, ByVal z As Single) As D3DVECTOR
Dim r1 As Single, r2 As Single
   If x = 0 And y = 0 And z = 0 Then Exit Function
   x = Deg2Rad(x)
   y = Deg2Rad(y)
   z = Deg2Rad(z)
   With Point
      .x = .x - Origin.x
      .y = .y - Origin.y
      .z = .z - Origin.z
      
      'y
      r1 = (.z * Cos(y)) - (.x * Sin(y))
      r2 = (.x * Cos(y)) + (.z * Sin(y))
      .x = r1
      .z = r2
      
      'x
      r1 = (.y * Cos(x)) - (.z * Sin(x))
      r2 = (.z * Cos(x)) + (.y * Sin(x))
      .y = r1
      .z = r2
      
      'z
      r1 = (.y * Cos(z)) - (.x * Sin(z))
      r2 = (.x * Cos(z)) + (.y * Sin(z))
      .y = r1
      .x = r2
      
      .x = .x + Origin.x
      .y = .y + Origin.y
      .z = .z + Origin.z
   End With
   SetRotation = Point
End Function
Public Sub MoveLeaves(TVTickCount As Long, Frequency As Single, Amplitude As Single, _
    Optional RandomOrientation As Boolean = False, _
    Optional Orientation As CONST_TV_LEAVES_ORIENTATION = TV_Leaves_XYZ, _
    Optional LeavesGroup As Long, Optional Recycle As Boolean = True)
Dim w As Single
Dim x As Single, y As Single, z As Single
Dim i As Long, j As Long
Dim m As Long
    Frequency = (Frequency / 2000)
    Amplitude = (Amplitude / 200)
    w = Amplitude * Sin(Frequency * TVTickCount)
    If RandomOrientation Then
        Orientation = CInt(Rnd * 6)
    End If
    Select Case Orientation
      Case Is = 0: x = w: y = 0: z = 0
      Case Is = 1: x = 0: y = w: z = 0
      Case Is = 2: x = 0: y = 0: z = w
      Case Is = 3: x = w: y = w: z = 0
      Case Is = 4: x = 0: y = w: z = w
      Case Is = 5: x = w: y = 0: z = w
      Case Is = 6: x = w: y = w: z = w
    End Select
    If Not LeavesGroup Then
        For j = 1 To UBound(LeavesV)
            With LeavesV(j)
                For i = 0 To .e - 1 Step 6
                    Leaves.MoveVertexInFace i, j + 0, x, y, z, True
                    Leaves.MoveVertexInFace i + 3, j + 0, x, y, z, True
                Next
            End With
            For m = UBound(LeafV) To 1 Step -1
               If LeafV(m).Parent = j Then
                  LeafMovement m, TVTickCount, Frequency, Amplitude
               End If
            Next
        Next
    Else
        With LeavesV(LeavesGroup)
            For i = 0 To .e - 1 Step 6
                Leaves.MoveVertexInFace i, LeavesGroup + 0, x, y, z, True
                Leaves.MoveVertexInFace i + 3, LeavesGroup + 0, x, y, z, True
            Next
            For m = 1 To UBound(LeafV)
               If LeafV(m).Parent = LeavesGroup Then
                  LeafMovement m, TVTickCount, Frequency, Amplitude
               End If
            Next
        End With
    End If
End Sub
Private Sub LeafMovement(LeafGroup As Long, TVTickCount As Long, Freq As Single, Amp As Single)
Dim n As Integer
Dim p As Long
Dim f As Single
Dim s As Single
Dim CurV As D3DVECTOR
Dim px As Single, py As Single, pz As Single
Dim c As Double, d As Single, e As Single
Dim r As Single
Dim RestLeaf As Boolean
Dim b As Integer
   With LeafV(LeafGroup)
      With LeavesV(.Parent).Pos
         px = .x
         py = .y
         pz = .z
      End With
      r = (.End - .Start)
      s = (.Start / .End) / 4
      For n = .Start To .End
         f = Amp * Sin(Freq * TVTickCount)
         c = Rnd * (b * Sin(Freq * b))
         b = b + 1
         With Leaf
            CurV = .GetVertexPosition(0, n)
            If CurV.y < Land.GetHeight(CurV.x, CurV.z) Then
               c = c - py
               d = (CurV.x - px) * (Rnd * 1)
               e = (CurV.z - pz) * (Rnd * 1.2)
               d = (d * Cos(d))
               e = (e * Sin(e))
               LeafV(LeafGroup).RestLeaf = True
            Else
               d = 0
               e = 0
               LeafV(LeafGroup).RestLeaf = False
            End If
         End With
         r = (Rnd * r)
         If Not .RestLeaf Then
            For p = 0 To 3
               Leaf.MoveVertexInFace p, n, f + (r * s) - d, f - c, f + (r * s) - e, True
            Next
         Else
            If TVTickCount - .Delay > 100 Then
               .Delay = TVTickCount
               For p = 0 To 3
                  Leaf.MoveVertexInFace p, n, f + (r * s) - d, f - c, f + (r * s) - e, True
               Next
            End If
         End If
      Next
   End With
End Sub
Public Function AddLeaf(Texture As Long, Size As Single, Amount As Integer, LeavesGroup As Integer) As Long
   If Amount <= 0 Or LeavesGroup > UBound(LeavesV) Then
      AddLeaf = 0
      Exit Function
   End If
   Dim i As Long, j As Integer, k As Integer, m As Integer
   Dim G_START As Long, G_END As Long, GetStart As Boolean
   Dim LVec(0 To 3) As D3DVECTOR
   Dim w As Single
   Dim NewFace As Boolean
   m = 0
   F_INDEX = F_INDEX + 1
   ReDim Preserve LeafV(F_INDEX)
   With LeavesV(LeavesGroup)
      For j = 1 To Amount
         For i = 0 To .e - 1 Step 4
            For k = 0 To 3
               LVec(k) = Leaves.GetVertexPosition((i + k), LeavesGroup)
               LVec(k) = SetRotation(Vector(LVec(k).x, LVec(k).y - Size, LVec(k).z), _
               Vector(LVec(k).x, LVec(k).y, LVec(k).z), 90, 0, 0)
            Next
            If FirstLeaf Then
               G_END = Leaf.AddFace(Texture, True)
               NewFace = False
            Else
               G_END = 1
               FirstLeaf = True
               NewFace = True
               Leaf.SetTexture Texture, 1
            End If
            If Not GetStart Then G_START = G_END: GetStart = True
            w = (2 / Pi) * ((G_END * Rnd) + G_END)
            Leaf.AddFaceFromPoint Texture, _
            LVec(m).x + Size * Cos(1) * Cos(w), LVec(m).y - (Size / 2), LVec(m).z + Size * Sin(1) * Sin(w), _
            LVec(m).x + Size * Cos(2) * Cos(w), LVec(m).y + (Size / 2), LVec(m).z + Size * Sin(2) * Sin(w), _
            LVec(m).x + Size * Cos(3) * Cos(w), LVec(m).y - (Size / 2), LVec(m).z + Size * Sin(3) * Sin(w), _
            LVec(m).x + Size * Cos(4) * Cos(w), LVec(m).y + (Size / 2), LVec(m).z + Size * Sin(4) * Sin(w), _
            1, 1, False, False
            m = m + 1
            If m > 3 Then m = 0
         Next
      Next
   End With
   With LeafV(F_INDEX)
      .Amount = LeavesV(LeavesGroup).e * Amount
      .Parent = LeavesGroup
      .Group = F_INDEX
      .Start = G_START
      .End = G_END
      .Delay = 0
      .Size = Size
      .RestLeaf = False
   End With
   AddLeaf = F_INDEX
   'Debug.Print "Group Number: " & Leaf.GetGroupsNumber
End Function
Public Sub Render(Optional TreeGroup As Long, Optional LeavesGroup As Long)
   If TreeGroup Then Tree.RenderGroupRange TreeGroup, TreeGroup
   If LeavesGroup Then Leaves.RenderGroupRange LeavesGroup, LeavesGroup
   If Not TreeGroup And Not LeavesGroup Then
      Tree.Render
      Leaves.Render
      Leaf.Render
   End If
End Sub

